///////////////////////////////////////////////////////////////////////////////////////////
// program which finds the longest common subsequence of exact pattern matchings (LCS-EPM)
// EPM-finding algorithm by Rolf Backofen/Sven Siebert (2007)
// LCS-EPM algorithm  by Steffen Heyne/Sebastian Will/Rolf Backofen (2008)
///////////////////////////////////////////////////////////////////////////////////////////

#include"mcsEnsemble.h"

///////////////////////////////////////////////////////////////////////////////////////////
// help screen
///////////////////////////////////////////////////////////////////////////////////////////
void printHelp()
{
  cerr << endl;
  cerr << " (c) copyright by Steffen Heyne - 2008"<< endl;
  cerr << "  - The Longest Common Subsequence of Exact Pattern Matchings (LCS-EPM) algorithm"<< endl;
  cerr << endl << endl;
  //cerr << "Usage : 1) expaRNA <options> seq1 ACGUACGU \"((....))\" seq2 ACGUACGU \"(......)\"" << endl;
  cerr << "Usage : 1) expaRNA -L <options> *.fa"                                     << endl;
  cerr << "        2) expaRNA -L <options> *.fa *.align"                             << endl << endl;
  cerr << "   1) two sequences given in fasta format"							   << endl;
  cerr << "      if no structure data is available, the sequence is folded"     << endl;
  cerr << "      to an mfe structure with Vienna RNAfold."                       << endl <<endl;
  cerr << "   2) like 1) and *.align contains reference alignment (use option -A)"<< endl <<endl;
  cerr << "Main Options:"                                                         << endl << endl;
  cerr << "  -L    : Calculate the maximal common arrangment of EPMs (LCS-EPM)"  << endl << endl;
 // cerr << "  -C    : starts the local clustering algorithm with specific clustering options" << endl <<endl;
  cerr << "  -A    : determine similarity/differences of LCS-EPM with reference alignment"  << endl;
  cerr << "          via two colored postscript files" << endl;
  cerr << "  -v    : verbose output"  << endl << endl;
  cerr << "EPM Selection Options:"                                                             << endl << endl;
  cerr << "  -s#   : # minmal size of an EPM (gamma)"   << endl;
  cerr << "          2 is default and returns all EPMs with at least 2 nucleotides"<< endl << endl;;
  cerr << "  -n#   : # determines the maximal number of returned EPMs"       << endl;
  cerr << "          0 is default an returns all EPMs"                       << endl;
  cerr << "          1 is the largest EPM and so on..."          << endl << endl;
  /*cerr << "Clustering Options: (under development!!!)"                                                             << endl << endl;
  cerr << "  -t#   : the distance-treshold value for the clustering algorithm"         << endl << endl;
  cerr << "  -c#   : # selects the clustering methode:"       << endl;
  cerr << "          0 CLUSTER_MAX_1 - cluster pattern only to best cluster before (default)"<< endl;
  cerr << "          1 CLUSTER_MAX_2 - like 0, but best cluster could not be used again!"<< endl;
  cerr << "          2 CLUSTER_ALL   - cluster pattern to all clusters within t"   << endl << endl;
  cerr << "  -d#   : # selects the  distance methode:"       << endl;
  cerr << "          0 DIST_SEQUENCE     - shortest sequence between 2 patterns (defualt)" << endl;
  cerr << "          1 DIST_EQUAL        - distances do not differ more than q#" << endl << endl;
  cerr << "          2 DIST_SHORTESTPATH - shortest path (!) through the structure"  << endl << endl;
  cerr << "  -q#   : the distance-equal threshold for -d1" << endl << endl;*/
  cerr << "Output Options:"                                               			<< endl << endl;
  cerr << "  -o#   : write LCS-EPM into file <#> as alignment"                		<< endl << endl;
  cerr << "  -i#   : write LCS-EPM into file <#> as constraint input for LocARNA"   << endl << endl;
  cerr << "  -e#   : write LCS-EPM into file <#> as single EPMs"   					<< endl << endl;
  cerr << "  -p   : save LCS-EPM as colored postscript file"                  		<< endl << endl;
  cerr << "  -h    : this info"                                                  	<< endl << endl;
}

///////////////////////////////////////////////////////////////////////////////////////////
// main
///////////////////////////////////////////////////////////////////////////////////////////
int main(int argc,char* argv[])
{
  ///////////////////////////////////////////////////////////////////////////////
  // read command line options
  ///////////////////////////////////////////////////////////////////////////////

  int    		opt;
  ExpaRNAOptions	myOptions;

  while ((opt=getopt(argc,argv,"s:n:LApo:i:e:hv"))!=-1)
   switch(opt)
   {
      case 's':
            myOptions.minPatternSize = atoi(optarg);
            if (myOptions.minPatternSize<2)
            {
               cerr << endl << " Minimal pattern size not valid!" << endl << endl;
               exit(1);
            }
         break;
      case 'n':
		     myOptions.maxPatternNumber = atoi(optarg);
			    if (myOptions.maxPatternNumber<0)
             {
               cerr << endl << " The number of returned patterns is not valid!" << endl << endl;
	    			exit(1);
             }
         break;
      case 'L':
      		myOptions.calcLCSEPM = true;
      	break;
      case 'A':
                 	myOptions.alignColor = true;
              break;
      case 'p':
            myOptions.clusterSaveCounter = 1;
         break;
      case 'o':
    	  myOptions.outfile 	= optarg;
    	  myOptions.alignOut	= true;
         break;
      case 'i':
          	  myOptions.locarna_file 	= optarg;
          	  myOptions.locarnaOut 		= true;
         break;
      case 'e':
			  myOptions.epm_file = optarg;
              myOptions.epmOut	 = true;
         break;
      case 'h':
    	  printHelp();
    	  exit(0);
    	 break;
      case 'v':
    	  myOptions.verboseOut = true;
    	 break;
      case '?' :
    	  cerr << "Unrecognized option." << endl;
    	  printHelp();
    	  exit(1);
    	 break;
  	}

  	///////////////////////////////////////////////////////////////////////////////
  	// Input recognition
  	///////////////////////////////////////////////////////////////////////////////

	for(;optind<argc;optind++)
		myOptions.cmdLineValues.push_back(argv[optind]);

  	int size=myOptions.cmdLineValues.size();
  	if ((size!=4) && (size!=6) && (size!=1)&& (size!=2))
   	{
    	cerr << "Please use correct input format!" << endl;
    	printHelp();
		exit(1);
   }
  	///////////////////////////////////////////////////////////////////////////////
   //  load data into the main interface and generate the two molecules
   ///////////////////////////////////////////////////////////////////////////////
   mcsEnsemble myNewEnsemble(myOptions);

   ///////////////////////////////////////////////////////////////////////////////
   //  start the maximum common substructure algorithm with the provided options
   //  start the clustering algorithm for these patterns and provided options
   ///////////////////////////////////////////////////////////////////////////////
   myNewEnsemble.startALL();

   return(0);
};
